--******************
-- (C) 2013, Brent Ozar Unlimited. 
-- See http://BrentOzar.com/go/eula for the End User Licensing Agreement.

--WARNING: This script suitable only for test purposes.
--Do not run on production servers.
--******************


--------------------------
--Preparation
--Don't try this at work.
--------------------------
--use master;
--GO

--RESTORE DATABASE AdventureWorks FROM 
--	DISK=N'C:\MSSQL\Backup\AdventureWorks2012-Full Database Backup.bak'
--	WITH REPLACE, 
--	MOVE 'AdventureWorks2012_Data' TO N'C:\MSSQL\Data\AdventureWorks2012_Data.mdf',
--	MOVE 'AdventureWorks2012_Log' TO N'C:\MSSQL\Data\AdventureWorks2012_Log.ldf';
--GO

--ALTER DATABASE AdventureWorks SET RECOVERY SIMPLE;
--GO

--USE [AdventureWorks]
--GO
--EXEC dbo.sp_changedbowner @loginame = N'sa', @map = false
--GO


--DBCC FREEPROCCACHE;
--GO



--------------------------
--DEMO!!!
--------------------------

--We're testing in AdventureWorks sample DB from Microsoft.
USE AdventureWorks;
GO
SET NOCOUNT ON;
GO
SET STATISTICS TIME, IO ON;
GO




--------------------------
--Literals
--------------------------

--Compare the execution plans of two queries
--Turn on actual execution plans
--Run one at a time, then together

SELECT DISTINCT City
FROM Person.Address 
WHERE StateProvinceID=80;
GO

SELECT DISTINCT City
FROM Person.Address 
WHERE StateProvinceID=9;
GO





--Estimated # of rows for StateProvinceID 80 : 4
--Estimated # of rows for StateProvinceID 9 : 4,564



--------------------------
--What if this query were simpler?
--------------------------

--Compare the execution plans for these two queries
--The second one "auto-parameterizes"

SELECT DISTINCT City
FROM Person.Address 
WHERE StateProvinceID=80;
GO

SELECT City
FROM Person.Address 
WHERE StateProvinceID=80;
GO




--------------------------
--Local Variable
--------------------------

DECLARE @StateProvinceID int = 80;

SELECT DISTINCT City
FROM Person.Address 
WHERE StateProvinceID= @StateProvinceID;
GO


DECLARE @StateProvinceID int = 9;

SELECT DISTINCT City
FROM Person.Address 
WHERE StateProvinceID= @StateProvinceID;
GO



--Estimated number of rows for each : 265.054
--Hmm, where's that come from?
--More on that later






--------------------------
--Stored Procedure
--------------------------

IF OBJECT_ID('dbo.GetCities') IS NULL
	EXEC ('CREATE PROCEDURE dbo.GetCities AS RETURN 0')
GO

ALTER PROCEDURE dbo.GetCities
	@StateProvinceID int
AS
SELECT DISTINCT City 
FROM Person.Address 
WHERE StateProvinceID=@StateProvinceID;
GO



--Run separately
EXEC dbo.GetCities @StateProvinceID=80;
GO


EXEC dbo.GetCities @StateProvinceID=9;
GO





--Estimated number of rows for each : 4
--Hmm. That's correct for StateProvinceID 80
--Did it reuse it for the second query? Look at the XML to see.







--------------------------
--Recompile hints
--------------------------

--What about this?
EXEC dbo.GetCities @StateProvinceID=80 WITH RECOMPILE;
GO
EXEC dbo.GetCities @StateProvinceID=9 WITH RECOMPILE;
GO








--Or this...
ALTER PROCEDURE dbo.GetCities
	@StateProvinceID int
AS
SELECT DISTINCT City 
FROM Person.Address 
WHERE StateProvinceID=@StateProvinceID OPTION (RECOMPILE);
GO




EXEC dbo.GetCities @StateProvinceID=80;
GO
EXEC dbo.GetCities @StateProvinceID=9;
GO




--What's the downside?





--------------------------
--Optimize for VALUE
--------------------------

--I always want the "big plan"
ALTER PROCEDURE dbo.GetCities
	@StateProvinceID int
AS
SELECT DISTINCT City 
FROM Person.Address 
WHERE StateProvinceID=@StateProvinceID 
	OPTION (OPTIMIZE FOR (@StateProvinceID=9));
GO



EXEC dbo.GetCities @StateProvinceID=80;
GO
EXEC dbo.GetCities @StateProvinceID=9;
GO







--------------------------
--Optimize for UNKNOWN
--------------------------

--I always want the "Average plan"
ALTER PROCEDURE dbo.GetCities
	@StateProvinceID int
AS
SELECT DISTINCT City 
FROM Person.Address 
WHERE StateProvinceID=@StateProvinceID 
	OPTION (OPTIMIZE FOR UNKNOWN);
GO



EXEC dbo.GetCities @StateProvinceID=80;
GO
EXEC dbo.GetCities @StateProvinceID=9;
GO


--Estimated number of rows for each : 265.054
--This is just like we saw for local variables!





--Let's see what statistics were used
--http://sqlblog.com/blogs/paul_white/archive/2011/09/21/how-to-find-the-statistics-used-to-compile-an-execution-plan.aspx
DBCC FREEPROCCACHE;
GO

ALTER PROCEDURE dbo.GetCities
	@StateProvinceID int
AS
SELECT DISTINCT City 
FROM Person.Address 
WHERE StateProvinceID=@StateProvinceID 
	OPTION (OPTIMIZE FOR UNKNOWN,
	    QUERYTRACEON 3604,
		QUERYTRACEON 9204
	);
GO


EXEC dbo.GetCities @StateProvinceID=80;
GO


--Look at the stats on index id 4
--This is an index that leads on StateProvinceID
--It's not the index SQL Server chooses to use in the plan,
--But it still figures out it can use stats for it!
DBCC SHOW_STATISTICS ('Person.Address','IX_Address_StateProvinceID')


--# Rows = 19614
--All density (second result set) = 0.01351351

SELECT 0.01351351 * 19614  --265.05398514

--That's where the "anonymous" number comes from.





--------------------------
--Wrapping up
--------------------------

--Which is better-- the "sniffed" value
--Or the "mediocre" value?